/**
 * A class of static methods that are helpful for quickly creating arrays (for list testing)
 * @author Joel Ross
 * @version Sp14
 */
public class ArrayHelper
{
	/**
	 * Creates an array of numbers from 1 to n
	 * @param n the size of the array
	 * @return the array
	 */
	public static int[] arrayOf(int n)
	{
		int[] array = new int[n];
		for(int i=0; i<n; i++)
			array[i] = i+1;
		return array;
	}
	
    /**
     * Creates an array of random numbers
     * @param size the size of the array
     * @param max the maximum value of the entries in the array
     * @return the random array
     */
    public static int[] randomArray(int size, int max)
    {
        int[] array = new int[size];
        for(int i=0; i<array.length; i++)
            array[i] = (int)(Math.random()*max);
        return array;
    }
    
    /**
     * Creates an array of random numbers IN ORDER
     * @param size the size of the array
     * @param spread maximum distance between array entries (double the average distance)
     * @return the sorted random array
     */
    public static int[] orderedArray(int size, int spread)
    {
        Random random = new Random();
        int[] array = new int[size];
        array[0] = random.nextInt(spread);
        for(int i=1; i<array.length; i++)
            array[i] = array[i-1]+(int)(Math.random()*spread);
        return array;
    }
    
    /**
     * Creates an array of numbers from 1 to n, shuffles
     * @param n the size of the array
     * @return the array
     */
    public int[] shuffled(int n)
	{
		ArrayList<Integer> nums = new ArrayList<Integer>();
		for(int i=0; i<n; i++)
			nums.add(i+1);
		Collections.shuffle(nums); //built in shuffle, for Lists

		int[] array = new int[n];
		for(int i=0; i<n; i++)
			array[i] = nums.get(i);
		return array;
	}
