import java.util.ArrayList;

/**
 * A collection of methods with different efficiencies
 * 
 * @author Joel Ross
 * @version Sp14
 */
public class EfficiencyLabMethods
{
	/**
	 * Calculates the sum of numbers up to n
	 */
	public static int methodA(int n)
	{
		int sum = 0;
		for(int i=1; i<=n; i++)
		{
			sum += i;
		}
		return sum;
	}

	/**
	 * Calculates the sum of the factors of n
	 */
	public static int methodB(int n)
	{
		int sum = 0;
		for(int i=1; i<=n; i++)
		{
			if(n%i == 0)
				sum += i;
		}
		return sum;
	}

	/**
	 * Doesn't do much of anything, actually...
	 */
	public static void methodC(int n)
	{
		for(int s = 0; s < Math.PI*1000; s++)
		{
			for(int c = 0; c < Math.PI*1000; c++)
			{
				if(Math.sin(s) > Math.cos(c))
				{
					System.out.println("Spam!!");
				}
			}
		}
	}


	/**
	 * Calculates the largest item in a 2D array
	 */
	public static int methodD(int[][] array)
	{
		if(array.length == 0 || array[0].length == 0)
			throw new IllegalArgumentException("Array must not be empty");

		int max = array[0][0];
		for(int i=0; i<array.length; i++)
		{
			for(int j=0; i<array[i].length; j++)
			{
				if(array[i][j] > max)
					max = array[i][j];
			}
		}
		return max;
	}


	/**
	 * Performs a binary search of the array.
	 */
	public static boolean methodE(int[] array, int target)
	{
		for(int i=1; i<array.length; i++)
		{
			if(array[i-1] > array[i])
				throw new IllegalArgumentException("Array must be ascending order for this binary search");
		}
		
		int startBound = 0;
		int endBound = array.length;

		while(endBound - startBound > 1)
		{
			int middleIndex = (startBound+endBound)/2;
			if(target < array[middleIndex])
			{
				endBound = middleIndex;
			}
			else
			{
				startBound = middleIndex;
			}
		}
		return (array[startBound] == target);
	}

	
	/**
	 * Calculates the list of all possible "sub-arrays" (an array made up from the elements of the given array). Includes the empty list.
	 * 
	 * This one is really challenging; try using the debugger to track how big many operations are happening each run!
	 * Note that calling a constructor should be considered a method call.
	 */
	public static ArrayList<ArrayList<Integer>> methodF(int[] array)
	{
		ArrayList<ArrayList<Integer>> allSubsets = new ArrayList<ArrayList<Integer>>();
		allSubsets.add(new ArrayList<Integer>()); //add empty subset
		for(int num : array) //go through the items
		{
			ArrayList<ArrayList<Integer>> tempList = new ArrayList<ArrayList<Integer>>(); //make a new temp list of subsets
			for(ArrayList<Integer> subset : allSubsets) //go through all subsets we've found
			{
				tempList.add(subset); //add the existing subsets (things we've calculated) to the templist
				
				ArrayList<Integer> newSubset = new ArrayList<Integer>(subset); //this constructor copies the current subset over
				newSubset.add(num); //add the item to that subset
				tempList.add(newSubset); //add that new, bigger item to the temp list
			}
			allSubsets = tempList; //assign the temp list to be the real list
			
		}
		return allSubsets;
	}

}
